EXPERIMENT = 'experiment'
GROUP = 'group'
JOB = 'job'
NOTEBOOK = 'notebook'
TENSORBOARD = 'tensorboard'
BUILD = 'build'
PIPELINE = 'pipeline'
KINDS = {EXPERIMENT, GROUP, JOB, NOTEBOOK, TENSORBOARD, BUILD, PIPELINE}
